/* swffillstyle.c	- SWFFillStyle class
 * $Id: swffillstyle.c,v 1.1 2004/10/31 13:52:10 ikegami Exp $
 *
 * Copyright (C) 2004 IKEGAMI Daisuke <ikegami at madscientist dot jp>
 * All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 * 
 */

#include <ming.h>
#include "ruby.h"
#include "mingc.h"

VALUE rb_cSWFFillStyle;

void rb_free_SWFFillStyle(p)
	 struct RSWFFillStyle *p;
{
  /* nop */
  
  return;
}

void
Init_swffillstyle()
{
  rb_cSWFFillStyle = rb_define_class_under(rb_mMing, "SWFFillStyle", 
                                           rb_cObject);

  return;
}

/* need not?
   SWFMatrix SWFFillStyle_getMatrix(SWFFillStyle fill);
*/
